"""Test Consumer Classes

This module provides a special Consumer/Producer class, which will collect
information about the client itself. (for debugging)

Author: Thomas Low <thomas.low@cern.ch>
"""

import time, logging

import producer, consumer, util

from gcm.common import test, logger, testproducer, testconsumer

class ClientTester(testproducer.TestProducer, testconsumer.TestConsumer):

	testList = []

	def __init__(self, level=logging.ERROR, active=1):
		self.level = level
		self.active = int(active)
		self.started = time.time()
		self.event = None
		if self.active:
			self.getLogList = logger.setupListLogging(self.level)
			
	def consume(self, test):
		ClientTester.testList.append(test.testName)
		
	def run(self, event):
		self.event = event
		
	def cleanup(self):
		if self.active and self.event:
			ClientTester.testList.append("grid_cm_client_wn")
			t = test.Test("grid_cm_client_wn")
			t.testVersion = "1.0"
			t.hostname	= util.hostname
			t.hostid	= util.hostid
			t.executionUnit = "ClientTester"
			t.executionStarttime = repr(self.started)
			t.executionEndtime = repr(time.time())
			t.summary = round(time.time()-self.started,2)
			l = self.getLogList
			if l:
				t.data = "\n".join(l)
			t.testlist = str(ClientTester.testList)
			self.event(t)
