"""Test File Saver Class

This module provides a Class which saves the results of tests to a single file.

Author: Thomas Low <thomas.low@cern.ch>
"""

import logging, re
log = logging.getLogger(__name__)

from gcm.common.testconsumer import TestConsumer
from gcm.common.test import Test

class TestFileSaver(TestConsumer):
	"""This module saves test results to a file."""
	
	def __init__(self, 	path, testfilter = None):
		self.path = path
		self.testfilter = testfilter
		self.testlist = []
		if testfilter:
			self.testpattern = re.compile(testfilter)
		else:
			self.testpattern = None
		
	def consume(self, test):
		if not self.testpattern or self.testpattern.match(test.testName):
			self.testlist.append(test)
			
	def cleanup(self):
		#store tests to file
		f = None
		try:
			f = open(self.path, "w")
			for t in self.testlist:
				f.write("["+t.testName+"]\n")
				for k,v in t.items():
					if k != 'data':
						f.write("%s:%s\n" % (k,v))
		finally:
			if f:
				f.close()
