"""Python Producer Class

This module provides a Python-Producer Class to create new tests.

PyTester executes a set of python modules within a directory and forwards their
produces test classes. SignedPyTester uses a signed python file to do the same.

Author: Thomas Low <thomas.low@cern.ch>
"""

import sys, os, time, logging, os.path, tempfile, imp
log = logging.getLogger(__name__)

from gcm.common.testproducer import TestProducer
from gcm.common import test, tar, openssl
from gcm.common.test import Test

from gcm.clients.wn import util

class PyTester(TestProducer):

	def __init__(self, directory):
		self.directory = directory
		self.modulelist = []
		
	def _getModule(self, path):
		module = os.path.basename(path)[:-3]
		fp, name, smt = imp.find_module(module, [os.path.dirname(path)])
		try:
			m = imp.load_module("pytester_"+module, fp, name, smt)
		finally:
			fp.close()
		return m
		
	def setup(self):
		"""Loads the python test-classes from the directory."""
		#create list of modules
		for f in os.listdir(self.directory):
			path = os.path.join(self.directory,f)
			if f[-3:] == ".py" and os.access(path, os.F_OK) \
					and not os.access(path, os.X_OK):
				try:
					
					m = self._getModule(path)
				except Exception, err:
					log.exception(err)
					continue
				if m.__dict__.has_key('getTest'):
					self.modulelist.append(m)
		
		
	def run(self, event):
		for m in self.modulelist:
			try:
				starttime = time.time()
				t = m.getTest()
				log.warning("%s needed %.5f seconds" % (m.__name__, time.time()-starttime))	
				if t:
					t.executionCommand = m.__name__+".getTest()"
					t.executionUnit = "PyTester"
					t.executionEndtime = repr(time.time())
					t.executionStarttime = repr(starttime)
					t.hostname	= util.hostname
					t.hostid	= util.hostid
					event(t)
			except Exception, err:
				log.exception(err)	
			

				
class SignedPyTester(PyTester):
	
	def __init__(self, directory, certs, allowed):
		self.certs = certs
		self.allowed = allowed
		self.tmpDirs = []
		PyTester.__init__(directory)
				
	def setup(self):
		"""Looks for tar balls, extracts the files and creates test-classes."""
		pathlist = []
		for f in os.listdir(self.directory):
			path = os.path.join(self.directory,f)
			if f[-9:] == '.py.smime' and os.access(path, os.F_OK) \
					and not os.access(path, os.X_OK):
				pathlist.append(path)
				
		for path in pathlist:
			#try to verify file
			tmpFile = tempfile.mktemp()
			tmpDir  = openssl._mkdtemp(self.directory)
			self.tmpDirs.append(tmpDir)
			try:
				cert = openssl.verifyFile(path, tmpFile, self.certs)
				#find out subject and check against allowed
				subj = openssl.x509subject(cert)
				if subj == self.allowed:
					pass					
				else:
					raise Exception("signer is not allowed: %s" % subj)
			except Exception, err:
				log.error("%s : %s" % (os.path.basename(path), err))
			try: os.remove(tarFile)
			except: pass
			
	def cleanup(self):
		PyTester.cleanup(self)
		for t in self.tmpDirs:
			try:
				os.system('rm -rf  %s ' % t)
			except Exception, err:
				log.warning(err)
